@@ -21,6 +21,7 @@ class acbuf
         inline unsigned int size() const { return w-r;}
         //! Returns capacity
         inline unsigned int freecapa() const { return m_nCapacity-w; }
+        inline unsigned int totalcapa() const { return m_nCapacity; }
         //! Erase all characters after a certain position  
         inline void erase(size_t pos) { w=r+pos; if(r==w) clear(); }
         //! Invalidate data on the beginning, move reading position  
@@ -28,7 +29,7 @@ class acbuf
         //! Mark externally appended data as valid, move writing position
         inline void got(size_t count) { w+=count;}
         //! for the worst case, move data to beginning to make space to read a complete new header
-        inline void move() { if(0==r) return; memmove(m_buf, m_buf+r, w-r); w-=r; r=0; }
+        inline void move() { if(0==r) return; memmove(m_buf, m_buf+r, w+1-r); w-=r; r=0; }
         //! Return pointer where to append data
         inline char *wptr() { return m_buf+w;};
         //! Return pointer to read valid data from
@@ -39,12 +40,9 @@ class acbuf
         inline void clear() {w=r=0;}
         
         //! Allocate needed memory
-        bool init(unsigned int capa);
+        bool setsize(unsigned int capa);
         bool initFromFile(const char *path);
 
-        //! like move but also feels unused memory with zeroes
-        void flush() { move(); memset(m_buf+w, 0, m_nCapacity-w); }
-                
         /*!
          * Writes to a (non-blocking) file descriptor, cares about EAGAIN and updates
          * position indexes. Optional chunklen variable specifies
@@ -61,7 +59,7 @@ class acbuf
          * \param fd File descriptor
          * \return Number of read bytes, negative on failures, see read(2)
          */
-        int sysread(int fd);
+        int sysread(int fd, unsigned int maxlen=MAX_VAL(unsigned int));
 
 
     protected:
@@ -76,14 +74,13 @@ class acbuf
 
 /* This is a light-weight and less cumbersome implementation of ostringstream.
  *
- * What it also makes possible... using as string, using alternative add() operators
- * for strings which take optional length, to make the thing work faster with C-strings.
+ * What it also makes possible: use itself as a string, use alternative add() operators
+ * for strings which can also specify the length, and it runs faster with zero-terminated strings.
  */
 class tSS : public acbuf
 {
 public:
 // map char array to buffer pointer and size
-#define _SZ2PS(x) x, (sizeof(x)-1)
 	inline tSS & operator<<(const char *val) { return add(val); }
 	inline tSS & operator<<(const mstring& val) { return add(val); };
 	inline tSS & operator<<(const acbuf& val) { return add(val.rptr(), val.size()); };
@@ -95,6 +92,9 @@ public:
 	inline tSS & operator<<(unsigned long val) __tss_nbrfmt("%lu", "%lx", val);
 	inline tSS & operator<<(long long val) __tss_nbrfmt("%lld", "%llx", val);
 	inline tSS & operator<<(unsigned long long val) __tss_nbrfmt("%llu", "%llx", val);
+#ifdef DEBUG
+	inline tSS & operator<<(void* val) __tss_nbrfmt("ptr:%llu", "ptr:0x%llx", (long long unsigned) val);
+#endif
 
     enum fmtflags {	hex, dec };
     inline tSS & operator<<(fmtflags mode) { m_fmtmode=mode; return *this;}
@@ -104,17 +104,18 @@ public:
     inline const char * data() const { return rptr();}
 
     inline tSS() : m_fmtmode(dec){}
-    inline tSS(size_t sz) : m_fmtmode(dec) { init(sz); }
-    inline tSS(const tSS &src) : m_fmtmode(src.m_fmtmode) { add(src.data(), src.size()); }
+    inline tSS(size_t sz) : m_fmtmode(dec) { setsize(sz); }
+    inline tSS(const tSS &src) : acbuf(), m_fmtmode(src.m_fmtmode) { add(src.data(), src.size()); }
     tSS & addEscaped(const char *fmt);
     inline tSS & operator<<(const char c) { reserve(size()+1); *(wptr())=c; got(1); return *this;}
-
+    inline tSS & clean() { clear(); return *this;}
+    inline tSS & append(const char *data, size_t len) { add(data,len); return *this; }
 
 protected:
     char fmtbuf[22];
 	fmtflags m_fmtmode;
 	inline void reserve(size_t minCapa) { minCapa+=(r+1); if(m_nCapacity>=minCapa) return;
-	char *p=(char*)realloc(m_buf, MYSTD::max(m_nCapacity, minCapa*2));
+	char *p=(char*)realloc(m_buf, std::max(m_nCapacity, minCapa*2));
 	if(!p) throw std::bad_alloc(); m_nCapacity=minCapa*2; m_buf=p; }
 
 	inline tSS & add(const mstring& val) { return add(val.data(), val.size());}
@@ -126,6 +127,6 @@ protected:
 
 };
 
-void _AddFooter(tSS &msg);
+cmstring& GetFooter();
 
 #endif
